#ifndef MISC_WINDOW_STUFF_H
#define MISC_WINDOW_STUFF_H

#include <app/AppDefs.h>
#include <app/Application.h>
#include <app/Roster.h>
#include <app/MessageFilter.h>
#include <interface/TextView.h>

#include "queue/PortableQueue.h"

/* Subsclass of BTextView that automatically reflows the text when it is resized */
class ReflowingTextView : public BTextView
{
public:
   ReflowingTextView(BRect frame, const char *name, BRect textRect, uint32 resizeMask, uint32 flags = B_WILL_DRAW | B_PULSE_NEEDED)
      : BTextView(frame, name, textRect, resizeMask, flags)
   {
      // empty
   }

   virtual void AttachedToWindow()
   {
      BTextView::AttachedToWindow();
      FixTextRect();
   }
  
   virtual void FrameResized(float w, float h)
   {
      BTextView::FrameResized(w, h);
      FixTextRect();
   }
 
   virtual void MouseMoved(BPoint where, uint32 code, const BMessage * msg)
   {
      uint32 start, len;
      if (code == B_INSIDE_VIEW) 
#ifdef B_BEOS_VERSION_5
         SetViewCursor(GetURLAt(where, start, len) ? B_CURSOR_SYSTEM_DEFAULT : B_CURSOR_I_BEAM);
#else
         be_app->SetCursor(GetURLAt(where, start, len) ? B_HAND_CURSOR : B_I_BEAM_CURSOR);
#endif
      else BTextView::MouseMoved(where, code, msg);
   }

   virtual void MouseDown(BPoint where)
   {
      uint32 start, len;
      if (GetURLAt(where, start, len))
      {
          PortableString url;
          char * buf = new char[len+1];
          GetText(start, len, buf);
          buf[len] = '\0';
          (void) be_roster->Launch(strncmp(buf, "mailto:", 7) ? "text/html" : "text/x-email", 1, &buf);
          delete [] buf;
      }
      BTextView::MouseDown(where);
   }

   void FixTextRect()
   {
      BRect t(Frame());
      SetTextRect(t);
   }

   void Clear()
   {
      Delete(0, TextLength()-1);
      _urls.MakeEmpty();
   }

   void AddURLRegion(uint32 start, uint32 len)
   {
        uint64 s = start;
        uint64 l = len;
        _urls.AddTail((s<<32)|l);      
   }

private:
   bool GetURLAt(BPoint pt, uint32 & retStart, uint32 & retLen) const
   {
      uint32 offset = (uint32) OffsetAt(pt);
      for (int i=_urls.CountItems()-1; i>=0; i--)
      {
         uint64 code = _urls[i];
         uint32 start = (uint32) (code >> 32);
         uint32 len   = (uint32) (code & 0xFFFFFFFF);
         if ((offset >= start)&&(offset < start + len))
         {
            retStart = start;
            retLen = len;
            return true;
         }
      }
      return false;
   }

   PortableQueue<uint64> _urls;
};
      
#endif
