#include "support/PortableMacros.h"
#include "ShareStrings.h"

static int _language = LANGUAGE_ENGLISH;   // Everybody's favorite language!

static const char _englishShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'B',  // "Bye" to current connection
   'S',  // Shared dir
   'D',  // Download dir
   'L',  // clear Log
   'R',  // Reset Layout
   'Q',  // Quit
   'U',  // Full user Queries
   'M',  // Beep When Mentioned
   'P',  // Open private chat window
   'F',  // Toggle file logging
};

static const char * _englishStrings[NUM_STRINGS] = {
   "Anonymous",
   "Unknown",
   " is now connected.",
   " has disconnected.",
   "User #",
   " (a.k.a ",
   " is now known as ",
   "true",
   "false",
   "<unknown type>",
   "User ",
   " is downloading ",
   "Error writing to file ",
   "An Error Occurred.",
   "Queued (Local machine too busy)",
   "Queued (Remote machine too busy)",
   "Sent: ",
   "Rcvd: ",
   "/sec, ",
   "Awaiting callback...",
   "Connecting...",
   "Download complete",
   "Download aborted",
   "Couldn't connect",
   "Preparing to Upload",
   "Preparing to Download",
   "to",
   "from",
   "Error starting download--no accept port!",
   "Error starting delayed connect!",
   "Error starting delayed connect--no transceiver thread!",
   "Error, couldn't create connect thread! (Try quitting BeShare, then Restarting Networking)",
   "(Sharing %i local files)",
   "Couldn't find 'shared' subdirectory, file sharing disabled!",
   "\0011File Name",
   "\0012User",
   "\0013SessionID",
   "\0014Connection",
   "File",
   "Connect to server",
   "Disconnect",
   "Open Shared Folder",
   "Open Downloads Folder",
   "Clear Chat Log",
   "Reset Layout",
   "About BeShare",
   "Quit",
   "Attributes",
   "Settings",
   "Max Simultaneous Uploads",
   "Max Simultaneous Downloads",
   "No Limit",
   "Upload Bandwidth",
   "Log",
   "Uploads",
   "User Events",
   "Query Changes",
   "Full User Queries",
   "I'm Firewalled",
   "Query:",
   "Start Query",
   "Stop Query",
   "Server:",
   "User Name:",
   "Download Selected Files",
   "Clear Finished/Failed Transfers",
   "Remove Selected",
   "Chat",
   "Name",
   "ID",
   "BeShare is listening on port %i.  Select Connect from the File menu to begin!",
   "Couldn't start file-sharing server thread!  (Try Restarting Networking, then restarting BeShare)",
   "Can't download from user ",
   ", no connection information available.",
   "Can't download files from ",
   ", because both of us are behind firewalls.",
   "File download accept session for ",
   " failed to initialize!",
   "File download session to ",
   "Changed Query to",
   "Started Query",
   "Couldn't start ShareFileTransfer session to serve remote transfer request!  (Try Restarting Networking, then restarting BeShare)",
   "Your name has been changed to ",
   "by",
   "Unknown user ",
   ", message \"",
   "\" not sent.",
   "No target user specified in /msg",
   "Stopped Query",
   "System",
   "Warning",
   "Error",
   "Action",
   "Connect to ",
   " - Connecting to ",
   " - %lu results shown (",
   " - Connected to ",
   "Connection established.",
   "Connecting to server...",
   "You are no longer connected to the MUSCLE server.",
   "Connection to server failed.",
   "English",
   "Spanish",
   "Language %s selected.  Change will take effect after you restart BeShare.",
   "Language", 
   "German",
   "Kind",
   "File Size",
   "Files",
   "Dutch",
   "Examining Files...",
   "Available irc-style commands:",
   "send a private message",
   "change your user name",
   "do something",
   "synonym for /action",
   "clear the chat log",
   "start a new query",
   "stop the current query",
   "connect to a server",
   "disconnect from the server",
   "quit BeShare",
   "show this help text",
   "<name or session id> <text>",
   "<name>",
   "<action>",
   "[queryString]",
   "[serverName]",
   "Portuguese",
   "French",
   "Beep When Mentioned",
   "Max Simultaneous Uploads Per User",
   "Timestamps",
   "AutoClear Finished Downloads",
   "Max Simultaneous Downloads Per User",
   "Italian",
   "Login on Startup",
   "Private Messages",
   "Open Private Chat Window",
   "BeShare Private Chat Window",
   "Chat With:",
   "Error, no user specified!", 
   "Save Preset",
   "Restore Preset",
   "Error, unknown command",
   "Type /help for a list of available commands.",
   " - (%lu-%lu) of %lu results shown (",
   "Display",
   "Info Messages",
   "Warning Messages",
   "Error Messages",
   "Logging Enabled",
   "Logging Disabled",
};

static const char _spanishShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'E',  // "Bye" to current connection
   'P',  // Shared dir
   'D',  // Download dir
   'L',  // clear Log
   'R',  // Reset Layout
   'Q',  // Quit
   'U',  // Full user Queries
   'M',  // Beep When Mentioned
   'I',  // Open private chat window
   'F',  // Toggle file logging
};

// Submitted by: Phantom (socca@uol.com.ar)
static const char * _spanishStrings[NUM_STRINGS] = {
   "Anónimo",
   "Desconocido",
   " está conectado.",
   " está desconectado.",
   "Usuario #",
   " (a.k.a ",
   " es conocido como ",
   "verdadero",
   "falso",
   "<tipo desconocido>",
   "Usuario ",
   " está descargando ",
   "error al escribir el archivo ",
   "se produjo un error.",
   "Queued (maquina local está muy ocupada)",
   "Queued (maquina remota está muy ocupada)",
   "Enviado: ",
   "Recibido: ",
   "/seg, ",
   "Esperando respuesta...",
   "Conectando...",
   "Download completo",
   "Download abortado",
   "no pudo conectar",
   "preparado para upload",
   "preparado para download",
   "a",
   "de",
   "Error al comenzar download--no aceptar puerto!",
   "Error al comenzar la conexión con delay!",
   "Error al comenzar conexión con delay--no transmite hilo!",
   "Error, no se pudo crear conectar hilo!",
   "(Compartiendo %i archivos locales)",
   "No se pudo encontrar el subdirectorio 'shared', compartir archivos desabilitado!",
   "\0011Archivo",
   "\0012Usuario",
   "\0013ID de Sesión",
   "\0014Conexión",
   "Archivo",
   "Conectar al Servidor",
   "Desconectar",
   "Abrir Carpeta Compartida",
   "Abrir Carpeta de Downloads",
   "Limpiar Historial de Chat",
   "Resetear Ventana",
   "Acerca de Beshare",
   "Salir",
   "Atributos",
   "Opciones",
   "Máximo de Uploads Simultáneos",
   "Máximo de Downloads Simultáneos",
   "Sin Límite ",
   "Ancho de Banda Para Upload",
   "Historia",
   "Uploads",
   "Eventos de usuario",
   "Cambios de Query",
   "Búsqueda Completa de Usuario",
   "Tengo Firewall",
   "Búsqueda:",
   "Buscar",
   "Parar",
   "Servidor:",
   "Nombre:",
   "Download archivos elegidos",
   "Limpiar terminados/Transferencias erróneas",
   "Remover elegido",
   "Chat",
   "Nombre",
   "ID",
   "BeShare está escuchando en puerto %i.  Seleccionar Conectar del menu de archivo para comenzar!",
   "No pudo comenzar el hilo de ejecución para compartir archivos!",
   "No se pudo bajar desde el usuario",
   ",no hay información de la conexión.",
   "No puede bajar archivos de ",
   ", porque ustedes dos están detras de un firewall.",
   "Sesión de download aceptada para ",
   " error para iniciar!",
   "Sesión de download para ",
   "Cambiada la búsqueda a",
   "Comenzar búsqueda",
   "No pudo comenzar la sesión transferencia de archivos para servir el pedido remoto de tranferencia!",
   "Tu nombre ha cambiado a ",
   "por",
   "Usuario desconocido ",
   ", mensage \"",
   "\" no enviado.",
   "No hay un usuario designado en /msg",
   "Búsqueda parada",
   "Sistema",
   "Peligro",
   "Error",
   "Acción",
   "Conectar a ",
   " - Conectando a ",
   " - %lu resultados mostrados (",
   " - Conectado a ",
   "Conexión establecida.",
   "Conectando a servidor...",
   "No está conectado al servidor muscle.",
   "Conexión al servidor falló.",
   "Ingles",
   "Castellano",
   "Lenguage %s seleccionado. El cambio tomará efecto cuando reinicie BeShare",
   "Idioma",
   "Alemán",
   "Tipo",
   "Tamaño",
   "Archivos",
   "Holandés",
   "Examinando Archivos...",
   "Comandos de irc disponibles:",
   "enviar un mensaje privado",
   "cambia tu nombre de usuario",
   "haz algo",
   "sinónimo para /action",
   "limpiar el historial de chat",
   "comenzar una nueva búsqueda",
   "detener la búsqueda actual",
   "conectar al servidor",
   "desconectar del servidor",
   "salir de BeShare",
   "mostrar este texto de ayuda",
   "<nombre o id de sesión> <texto>",
   "<nombre>",
   "<acción>",
   "[búsqueda]",
   "[servidor]",
   "Portugues",
   "Francés",
   "Emite un Beep Cuando su Nombre es Mencionado",
   "Máximo de Uploads Simultáneos por Usuario",
   "Timestamps",
   "AutoLimpiar Downloads Terminados",
   "Máximo de Downloads Simultáneos por Usuario",
   "Italiano",
   "Conectar al Inicio",
   "Mensaje Privado",
   "Abrir Ventana Privada de Chat",
   "BeShare Ventana Privada de Chat",
   "Chatear Con:",
   "¡Error, usuario no especificado!",
   "Salvar Configuración",
   "Restaurar Configuración",
   "Error, comando desconocido",
   "Pulse /help para una lista de comandos disponibles.",
   " - (%lu-%lu) de %lu resultados mostrados (",
   "Mostrar",
   "Mensajes Informativos",
   "Mensajes de Aviso",
   "Mensajes de Error",
   "Historia Habilitado",
   "Historia Deshabilitado",
};


static const char _germanShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'T',  // "Bye" to current connection
   'F',  // Shared dir
   'D',  // Download dir
   'L',  // clear Log
   'Z',  // Reset Layout
   'Q',  // Quit
   'K',  // Full user Queries
   'M',  // Beep When Mentioned
   'P',  // Open private chat window
   'G',  // Toggle file logging
};

// Submitted by:  hoshy (beat.rupp@gmx.net)
static const char * _germanStrings[NUM_STRINGS] = {
   "anonym",
   "unbekannt",
   " ist jetzt verbunden.",
   " hat die Verbindung getrennt.",
   "Benutzer #",
   " (alias ",
   " heisst jetzt ",
   "wahr",
   "falsch",
   "<unbekannter Typ>",
   "Benutzer ",
   " lädt herunter ",
   "Fehler beim Schreiben der Datei ",
   "Es trat ein Fehler auf.",
   "In Warteschlange (Lokaler Rechner ist überlastet)",
   "In Warteschlange (Entfernter Rechner ist überlastet)",
   "Gesendet: ",
   "Empfangen: ",
   "/Sek., ",
   "Erwarte Rückruf...",
   "Verbinde...",
   "Download beendet",
   "Download abgebrochen",
   "Konnte nicht verbinden",
   "Bereite Upload vor",
   "Bereite Download vor",
   "an",
   "von",
   "Fehler beim Starten des Downloads--keine gültigen Ports!",
   "Fehler beim Starten einer wartenden Verbindung!",
   "Error starting delayed connect--no transceiver thread!",
   "Fehler beim Starten einer wartenden Verbindung - kein Übermittlungs-Thread vorhanden!",
   "(Es sind %i lokale Dateien freigegeben)",
   "Konnte Ordner 'shared' nicht finden. Dateifreigabe deaktiviert!",
   "\0011Dateiname",
   "\0012Benutzer",
   "\0013SitzungsID",
   "\0014Verbindung",
   "Datei",
   "Verbinde mit Server",
   "Verbindung trennen",
   "Freigegebenen Ordner öffnen",
   "Downloads-Ordner öffnen",
   "Chat-Aufzeichnung Löschen",
   "Ansicht Zurücksetzen",
   "Über BeShare",
   "Beenden",
   "Attribute",
   "Einstellungen",
   "Max. Gleichzeitige Uploads",
   "Max. Gleichzeitige Downloads",
   "Keine Beschränkung",
   "Upload-Bandbreite",
   "Aufzeichnung",
   "Uploads",
   "Benutzerereignisse",
   "Änderungen abfragen",
   "Vollständige Benutzerabfragen",
   "Ich Bin Hinter Einer Firewall",
   "Abfrage:",
   "Starte Abfrage",
   "Stoppe Abfrage",
   "Server:",
   "Benutzername:",
   "Markierte Dateien herunterladen",
   "Beendete/Fehlerhafte Übertragungen löschen",
   "Markierte entfernen",
   "Chat",
   "Name",
   "ID",
   "BeShare lauscht auf Port %i.  Wählen Sie Verbinden im Menu Datei, um loszulegen!",
   "Server für die Dateifreigabe konnte nicht gestartet werden!",
   "Kann nicht herunterladen vom Benutzer ",
   ", keine Informationen zur Verbindung sind vorhanden.",
   "Kann keine Dateien herunterladen von ",
   ", weil wir beide hinter einer Firewall sind.",
   "Sitzung für eingehenden Dateidownload von",
   " initialisieren schlug fehl!",
   "Sitzung für Dateidownload an ",
   "Abfrage geändert in",
   "Abfrage gestartet",
   "Download-Sitzung für anfragenden, entfernten Rechner konnte nicht gestartet werden!",
   "Ihr Name wurde geändert in ",
   "von",
   "Unbekannter Benutzer ",
   ", Mitteilung \"",
   "\" nicht gesendet.",
   "Kein Zielbenutzer in /msg angegeben",
   "Abfrage gestoppt",
   "System",
   "Warnung",
   "Fehler",
   "Aktion",
   "Verbinde mit ",
   " - Verbinde mit ",
   " - %lu resultate angezeigt (",
   " - Verbunden mit ",
   "Verbindung hergestellt.",
   "Verbinde mit Server...",
   "Sie sind nicht mehr mit dem MUSCLE-Server verbunden.",
   "Verbindung zum Server schlug fehl.",
   "Englisch",
   "Spanisch",
   "Sprache %s ausgewählt.  Sie müssen BeShare neu starten, damit die Änderungen wirksam werden.",
   "Sprache",
   "Deutsch",
   "Art",
   "Dateigrösse",
   "Dateien",
   "Holländisch",
   "überprüfe Dateien",
   "Vorhandene IRC-Kommandos:",
   "eine private Nachricht senden",
   "ändern Sie Ihren Benutzernamen",
   "machen Sie etwas",
   "Synonym für /action",
   "Chat-Aufzeichnung löschen",
   "neue Abfrage starten",
   "aktuelle Abfrage stoppen",
   "verbinde mit Server",
   "Verbindung trennen",
   "BeShare beenden",
   "diesen Hilfetext anzeigen",
   "<Name oder SitzungsID> <text>",
   "<Name>",
   "<Aktion>",
   "[AbfrageString]",
   "[ServerName]",
   "Portugiesisch",
   "Französisch",
   "Piepston Ausgeben, Wenn Sie Angesprochen Werden",
   "Max. Gleichzeitige Uploads pro Benutzer",
   "Zeitstempel",
   "Beendete Downloads Automatisch Löschen",
   "Max. Gleichzeitige Downloads pro Benutzer",
   "Italienische",
   "Beim Start Einloggen",
   "Private Nachrichten",
   "Privates Chat-Fenster öffnen",
   "Privates BeShare Chat-Fenster",
   "Chatten mit:",
   "Fehler, Kein Benutzer Angegeben!",
   "Standard-Einstellung Speichern",
   "Standard-Einstellung Wiederherstellen",
   "Fehler, unbekannter Befehl",
   "Geben Sie /help ein, um eine Liste von verfügbaren Befehlen anzuzeigen.",
   " - (%lu-%lu) von %lu resultate angezeigt (",
   "Anzeige",
   "Informationsmeldungen",
   "Warnungsmeldungen",
   "Fehlermeldungen",
   "Protokollierung Ein",
   "Protokollierung Aus",
};

static const char _dutchShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'B',  // "Bye" to current connection
   'G',  // Shared dir
   'I',  // Download dir
   'L',  // clear Log
   'H',  // Reset Layout
   'Q',  // Quit
   'Z',  // Full user Queries
   'M',  // Beep When Mentioned
   'P',  // Open private chat window
   'F',  // Toggle file logging
};

// Submitted by Zathros/Ted Leeuwesteijn Ted@nds.nl, 3rdrock@bart.nl
static const char * _dutchStrings[NUM_STRINGS] = {
   "Anoniem",
   "onbekend",
   " is nu verbonden.",
   " heeft verbroken.",
   "gebruiker #",
   " (o.b.a ",
   " heeft nu de naam ",
   "waar",
   "onecht",
   "<onbekende soort>",
   "Gebruiker ",
   " is aan het binnenhalen ",
   "Fout tijdens schrijf operatie ",
   "Er is een fout opgetreden.",
   "Pauze (Lokale machine heeft het te druk)",
   "Pauze (Andere machine heeft het te druk)",
   "Zend: ",
   "Ontvangen: ",
   "/sec, ",
   "Wacht op terugbel bevestiging...",
   "Verbinden...",
   "Binnenhalen klaar",
   "Binnenhalen geannuleerd",
   "Kon niet verbinden",
   "Opmaken voor sturen",
   "Opmaken voor binnenhalen",
   "naar",
   "van",
   "Fout bij het binnenhalen bestand--geen acceptatie poort!",
   "Fout bij vertraagde verbinding!",
   "Fout bij vertraagde verbinding--geen transceiver draad!",
   "Fout, kan geen verbindings draad vinden!",
   "(Delen van %i lokale bestanden)",
   "Kan geen 'gedeelde' bestanden vinden in de shared folder, Delen niet geactiveerd!",
   "\0011Bestands Naam",
   "\0012Gebruiker",
   "\0013SessieNr",
   "\0014Verbinding",
   "Bestand",
   "Verbinden met server",
   "Verbreken",
   "Open Gedeelde Folder",
   "Open Inkomende Folder",
   "Maak Chat Log Schoon",
   "Herstel Venster",
   "Over BeShare",
   "Einde",
   "Attributen",
   "Instellingen",
   "Max Simultane Uploads",
   "Max Simultane Downloads",
   "Onbegrensd",
   "Uitgaande Bandbreedte",
   "Log",
   "Inkomend",
   "Gebruikers Gebeurtenissen",
   "Veranderde Zoekopdracht",
   "Volledige Gebruiker Zoekopdrachten",
   "Ik Ben Afgeschermd",
   "Zoek:",
   "Start Zoek",
   "Stop Zoek",
   "Server:",
   "Gebruikers Naam:",
   "Haal geselecteerde bestanden op",
   "Maak beeindigde/mislukte Transmissies schoon",
   "Verwijder Selectie",
   "Praat",
   "Naam",
   "Nr",
   "BeShare luistert op poort %i. Selecteer verbinden in het bestands menu om te beginnen!",
   "Kan geen BestandsDelingServerDraad vinden!",
   "Kan niet binnenhalen van de gebruiker ", 
   ", geen verbindings informatie beschikbaar.",
   "Kan niet binnenhalen van ",
   ", omdat wij beiden achter een firewall zitten.",
   "Bestand binnenhalen acceptatie voor ",
   " fout in initialisatie!",
   "Bestands verzoek sessie naar",
   "Zoek opdracht veranderd in",
   "Zoek opdracht gestart",
   "Kan geen GedeeldeBestandsOverdracht sessie starten voor extern overdracht verzoek!",
   "Je naam is veranderd in ",
   "door",
   "Onbekende gebruiker ",
   ", bericht \"",
   "\" niet verzonden.",
   "Geen gebruikersnaam geselecteerd in /msg",
   "Zoek opdracht gestopt",
   "Systeem",
   "Waarschuwing",
   "fout",
   "Aktie",
   "Verbind met ",
   " - Verbinden met ",
   " - toont %lu resultaten (",
   " - Verbonden met ",
   "Verbinding opgebouwd.",
   "Verbinden met server...",
   "U bent niet langer verbonden met de MUSCLE server.",
   "Verbinding met server is mislukt.",
   "Engels",
   "Spaans",
   "Taal %s geselekteerd. De veranderingen vinden plaats na het herstarten van BeShare.",
   "Taal", 
   "Duits",
   "Soort",
   "Bestands Grootte",
   "Aantal",
   "Nederlands",
   "Onderzoekt bestanden",
   "Beschikbare irc-stijl commandos:",
   "zend een prive bericht",
   "verander je gebruikersnaam",
   "Doe iets",
   "synoniem voor /action",
   "maak praat venster schoon",
   "start een nieuwe zoek aktie",
   "stop de huidige zoekaktie",
   "verbind met een server",
   "verbreek met de huidige server",
   "einde BeShare",
   "geeft deze helptekst weer",
   "<naam of sessie_Nr> <Tekst>",
   "<eigennaam>",
   "<aktie>",
   "[zoekregel]",
   "[servernaam]",
   "Portugees",
   "Frans",
   "Geef Geluid Wanneer u Genoemd Word",
   "Max Simultane Uploads per Gebruiker",
   "Toon Tijd",
   "AutoBeeindigde Downloads Schoon",
   "Max Simultane Downloads per Gebruiker",
   "Italians",
   "Verbind bij Starten",
   "Prive Berichten",
   "Open Prive Venster",
   "BeShare Prive Venster",
   "Praat Met:",
   "Fout, Geen Gebruiker Geselecteerd",
   "Bewaar Instelling",
   "Herstel Instelling",
   "Fout, onbekende opdracht",
   "Gebruik /help voor een lijst van beschikbare opdrachten.",
   " - (%lu-%lu) van %lu toont resultaten (",
   "Toon",
   "Berichten Info",
   "Alarm Berichten",
   "Fout Berichten",
   "Logboek Aan",
   "Logboek Uit",
};

static const char _portugueseShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'B',  // "Bye" to current connection
   'S',  // Shared dir
   'D',  // Download dir
   'L',  // clear Log
   'R',  // Reset Layout
   'Q',  // Quit
   'U',  // Full user Queries
   'M',  // Beep When Mentioned
   'P',  // Open private chat window
   'F',  // Toggle file logging
};

// Submitted by: BGA (bga@din.uem.br)
static const char * _portugueseStrings[NUM_STRINGS] = {
   "Anônimo", 
   "Desconhecido", 
   " conectou-se.", 
   " desconectou-se.", 
   "Usuário #", 
   " (t.c.c ", 
   " agora é conhecido por ", 
   "verdadeiro", 
   "falso", 
   "<tipo desconhecido>", 
   "Usuário ", 
   " está transferindo ", 
   "Erro ao escrever no arquivo ", 
   "Um Erro Ocorreu.", 
   "Empilhado (Máquina local muito ocupada)", 
   "Empilhado (Máquina remota muito ocupada)", 
   "Enviado: ", 
   "Recebido: ", 
   "/seg, ", 
   "Esperando retorno...", 
   "Conectando...", 
   "Transferência completa", 
   "Transferência abortada", 
   "Impossivel conectar", 
   "Preparando Transferência", 
   "Preparando Transferência", 
   "para", 
   "de", 
   "Erro ao iniciar transferência--porta accept inexistente!", 
   "Erro ao iniciar conexão adiada!", 
   "Erro ao iniciar conexão adiada--thread transceiver inexistente!", 
   "Erro, impossivel criar thread de conexão!", 
   "(Compartilhando %i arquivos locais)", 
   "Impossivel encontrar subdiretório 'shared', compartilhamento de arquivos desabilitado!", 
   "\0011Nome do Arquivo", 
   "\0012Usuário", 
   "\0013ID de Sessão", 
   "\0014Conexão", 
   "Arquivo", 
   "Conectar ao servidor", 
   "Desconectar", 
   "Abrir Diretório Shared", 
   "Abrir Diretório Downloads", 
   "Apagar Histórico de Chat", 
   "Resetar Layout", 
   "Sobre BeShare", 
   "Sair", 
   "Atributos", 
   "Configurações", 
   "Max de Uploads Simultâneos", 
   "Max de Downloads Simultâneos", 
   "Sem Limite", 
   "Banda de Transferência (Upload)", 
   "Histórico", 
   "Transferências (Uploads)", 
   "Eventos de Usuários", 
   "Mudanças de Pesquisa", 
   "Pesquisas Completas por Usuários", 
   "Estou Atrás de um Firewall", 
   "Pesquisa:", 
   "Iniciar Pesquisa", 
   "Parar Pesquisa", 
   "Servidor:", 
   "Nome de Usuário:", 
   "Transferir Arquivos Selecionados", 
   "Limpar Transferências Completas/Falhas", 
   "Remover Selecionados", 
   "Conversação", 
   "Nome", 
   "ID",                                              
   "BeShare está escutando na porta %i.  Selecione Conectar no menu Arquivo para iniciar!",  
   "Impossivel iniciar thread do servidor de compartilhamento de arquivos!", 
   "impossível transferir do usuário ", 
   ", informação sobre conexão não disponível.", 
   "Impossível transferir arquivos de ", 
   ", porque ambos estamos atrás de firewalls.", 
   "Sessão de aceitação de transferência de arquivo para ", 
   " falhou durante a inicialização!", 
   "Sessão de transferência de arquivo para ", 
   "Pesquisa Mudada para", 
   "Pesquisa Iniciada", 
   "Impossível iniciar sessão de Tranferência de Arquivo para servir requisição remota de transferência!", 
   "Seu nome foi mudado para ", 
   "por", 
   "Usuário desconhecido ", 
   ", mensagem \"", 
   "\" não enviada.", 
   "Nenhum usuário destino especificado no /msg", 
   "Pesquisa Interrompida", 
   "Sistema", 
   "Aviso", 
   "Erro", 
   "Ação", 
   "Conectar em ", 
   " - Conectando em ", 
   " - %lu resultados mostrados (", 
   " - Conectado a ", 
   "Conexão estabelecida.", 
   "Conectando ao servidor...", 
   "Você não está mais conectado ao servidor MUSCLE.", 
   "Conexão ao servidor falhou.", 
   "Inglês", 
   "Espanhol", 
   "Lingua %s selecionada.  A mudança se tornará efetiva após você reiniciar o BeShare.",  
   "Lingua", 
   "Alemão", 
   "Tipo", 
   "Tamanho do Arquivo", 
   "Arquivos", 
   "Holandês", 
   "Examinando Arquivos...", 
   "Comandos estilo irc disponíveis:", 
   "envia uma mensagem privada", 
   "muda seu nome de usuário", 
   "faz alguma coisa", 
   "sinônimo para /action", 
   "limpa o histórico de chat", 
   "inicia uma nova pesquisa", 
   "finaliza a pesquisa atual", 
   "conecta a um servidor", 
   "desconecta de um servidor", 
   "sai do BeShare", 
   "mostra esse texto de ajuda", 
   "<nome ou id de sessão> <texto>", 
   "<nome>", 
   "<ação>", 
   "[stringDeBusca]", 
   "[nomeDoServidor]",
   "Português",
   "Francés",
   "Emitir um Bip Quando Seu Nome for Mencionado",
   "Max de Uploads Simultâneos por Usuário",
   "Timestamps",
   "AutoLimpar Downloads Completos",
   "Max de Downloads Simultâneos por Usuário",
   "Italiano",
   "Conectar ao Iniciar",
   "Mensagens Privadas",
   "Abrir Janela de Conversação Privada",
   "Janela de Conversação Privada do BeShare",
   "Conversar com:",
   "Erro, nenhum usuário especificado!",
   "Salvar Configuração",
   "Restaurar Configuração",
   "Erro, comando desconhecido",
   "Digite /help para obter uma lista dos comandos disponíveis.",
   " - (%lu-%lu) dos %lu resultados mostrados (",
   "Mostrar",
   "Mensagens Informativas",
   "Mensagens de Aviso",
   "Mensagens de Erro",
   "Log Habilitado",
   "Log Desabilitado",
}; 

static const char _frenchShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'B',  // "Bye" to current connection
   'P',  // Shared dir
   'T',  // Download dir
   'E',  // clear Log
   'R',  // Reset Layout
   'Q',  // Quit
   'U',  // Full user Queries
   'M',  // Beep When Mentioned
   'I',  // Open private chat window
   'F',  // Toggle file logging
};


/* French translation by Dider Bertrand (didix@belgacom.net) */
static const char * _frenchStrings[NUM_STRINGS] = {
   "Anonyme",
   "Inconnu",
   " connecté.",
   " déconnecté.",
   "Utilisateur #",
   " (alias ",
   " maintenant connu comme ",
   "vrai",
   "faux",
   "<type inconnu>",
   "Utilisateur ",
   " en téléchargement ",
   "Erreur d'écriture de fichier ",
   "Une erreur s'est produite.",
   "En attente (Machine locale trop occupée)",
   "En attente (Machine distante trop occupée)",
   "Envoyé: ",
   "Reçu: ",
   "/sec, ",
   "En attente d'une réponse...",
   "Connexion...",
   "Téléchargement terminé",
   "Téléchargement abandonné",
   "Connexion impossible",
   "Préparation à l'envoie",
   "Préparation au téléchargement",
   "à",
   "de",
   "Erreur en commençant le téléchargement--pas d'acceptation du port!",
   "Erreur à l'établissement de la connexion retardée!",
   "Erreur à l'établissement de la connexion retardée--pas de thread transmetteur!",
   "Erreur, création de thread connecteur impossible!",
   "(Partage de %i fichiers locaux)",
   "Dossier 'Shared' introuvable, fichiers partagés indisponibles!",
   "\0011Nom du Fichier",
   "\0012Utilisateur",
   "\0013N° de Session",
   "\0014Connexion",
   "Fichier",
   "Connecter au serveur",
   "Déconnecter",
   "Ouvrir le Dossier Partagé",
   "Ouvrir le Dossier des Téléchargements",
   "Effacer l'historique des Discussions",
   "Restaurer la Fenêtre",
   "A Propos de BeShare",
   "Quitter",
   "Attributs",
   "Configuration",
   "Max. d'envois Simultanés",
   "Max. de Téléchargements Simultanés",
   "Illimité",
   "Type de Connexion",
   "Historique",
   "Envois",
   "Evénements Utilisateur",
   "Changements de Recherche",
   "Tout Rechercher chez l'utilisateur",
   "Je Suis Protégé par un Firewall",
   "Requête:",
   "Démarrer Requête",
   "Arrêter Requête",
   "Serveur:",
   "Nom d'utilisateur:",
   "Télécharger les fichiers sélectionnés",
   "Effacer les transferts terminés/abandonnés",
   "Effacer la sélection",
   "Discussion",
   "Nom",
   "N°",
   "BeShare utilise le port %i.  Sélectionnez Connecter du menu Fichier pour commencer!",
   "Thread du serveur d'échange de fichiers impossible à lancer!",
   "Téléchargement impossible depuis l'utilisateur ",
   ", aucune information de connexion disponible.",
   "Téléchargement de fichiers impossible depuis ",
   ", parce que nous sommes derrière des Firewalls.",
   "Acceptation de session de téléchargement de fichier pour",
   " initialisation ratée!",
   "Session de téléchargement de fichier à ",
   "Requête changée en",
   "Requête démarrée",
   "Session PartageFichierTransfert impossible à commencer pour répondre à une requête de transfert distante!",
   "Votre nom a été changé en ",
   "par",
   "Utilisateur inconnu : ",
   "; le message \"",
   "\" n'a pas été envoyé.",
   "Pas de destinataire spécifié dans /msg",
   "Requête arrêtée",
   "Système",
   "Avertissement",
   "Erreur",
   "Action",
   "Connecter à ",
   " - Connexion à ",
   " - %lu résultats affichés (",
   " - Connecté à ",
   "Connexion établie.",
   "Connexion au serveur...",
   "Vous n'êtes plus connecté au serveur MUSCLE.",
   "Connexion au serveur ratée.",
   "Anglais",
   "Espagnol",
   "Langue %s sélectionnée.  Le changement prendra effet après avoir redémarré BeShare.",
   "Langue",
   "Allemand",
   "Type",
   "Taille du Fichier",
   "Fichiers",
   "Néerlandais",
   "Contrôle des fichiers...",
   "Commandes du type irc disponibles:",
   "envoyer un message privé",
   "changer votre nom d'utilisateur",
   "faire quelque chose",
   "synonyme pour /action",
   "effacer l'historique de discussion",
   "démarrer une nouvelle recherche",
   "arrêter la recherche actuelle",
   "connecter à un serveur",
   "déconnecter du serveur",
   "quitter BeShare",
   "afficher ce texte d'aide",
   "<nom ou n° de session> <texte>",
   "<nom>",
   "<action>",
   "[Chaîne de recherche]",
   "[Nom du serveur]",
   "Portugais",
   "Français",
   "Faire un Bip une Fois Mentionné",
   "Max. d'envois Simultanés par Utilisateur",
   "Horodateurs",
   "AutoEffacer les Transferts Terminés",
   "Max. de Téléchargements Simultanés par Utilisateur",
   "Italien",
   "Connexion au Lancement",
   "Messages Privés",
   "Ouvrir une Fenêtre de Discussion Privée",
   "Fenêtre Privée de Discussion de BeShare",
   "Discussion avec:",
   "Erreur, Aucun Utilisateur Spécifié!",
   "Enregistrer la Configuration",
   "Restaurer la Configuration",
   "Erreur, commande inconnue",
   "Tapez /help pour obtenir la liste des commandes disponibles.",
   " - (%lu-%lu) des %lu résultats affichés (",
   "Affichage",
   "Messages d'Information",
   "Messages d'Avertissement",
   "Messages d'Erreur",
   "Historique Disponible",
   "Historique Indisponible",
};


// DA@Y2K: Added Italian localisation.
static const char _italianShortcuts[NUM_SHORTCUTS] = {
   'N',  // New connection
   'B',  // "Bye" to current connection
   'S',  // Shared dir
   'D',  // Download dir
   'L',  // clear Log
   'R',  // Reset Layout
   'Q',  // Quit
   'U',  // Full user Queries
   'M',  // Beep When Mentioned
   'I',  // Open private chat window
   'F',  // Toggle file logging
};

// Submitted by Dario Accornero <bexoft@bexoft.com>
static const char * _italianStrings[NUM_STRINGS] = {
   "Anonimo",
   "Ignoto",
   " è ora connesso.",
   " si è disconnesso.",
   "Utente #",
   " (ossia ",
   " è ora noto come ",
   "vero",
   "falso",
   "<tipo ingnoto>",
   "Utente ",
   " sta scaricando ",
   "Errore nella scrittura del file ",
   "Si è verificato un errore.",
   "In coda (Macchina locale sovraccarica)",
   "In coda (Macchina remota sovraccarica)",
   "Inv.: ",
   "Ric.: ",
   "/sec, ",
   "In attesa di risposta...",
   "Collegamento...",
   "Scaricamento completo",
   "Scaricamento interrotto",
   "Impossibile connettersi",
   "Preparazione al caricamento",
   "Preparazione allo scaricamento",
   "a",
   "da",
   "Errore nell'iniziare lo scaricamento--nessuna porta accettata!",
   "Errore nell'iniziare la connessione procrastinata!",
   "Errore nell'iniziare la connessione procrastinata--nessun thread di ricezione!",
   "Errore, impossibile creare thread di connessione! (Prova ad uscire da BeShare, e poi a far ripartire la rete)",
   "(Sto condividendo %i files locali)",
   "Impossibile trova la cartella 'shared', condivisione disabilitata!",
   "\0011Nome File",
   "\0012Utente",
   "\0013ID Sessione",
   "\0014Connessione",
   "File",
   "Collegati al server",
   "Disconnetti",
   "Apri Cartella Condivisa",
   "Apri Cartella Scaricamento",
   "Cancella il Chat Log",
   "Ripristina Impostazioni Finestra",
   "Informazioni su BeShare",
   "Esci",
   "Attributi",
   "Impostazioni",
   "Max Caricamenti Simultanei",
   "Max Scaricamenti Simultanei",
   "Nessun Limite",
   "Velocità di Caricamento",
   "Log",
   "Caricamenti",
   "Eventi Utente",
   "Cambiamenti della Ricerca",
   "Ricerche Utente Complete",
   "Sono Dietro un Firewall",
   "Ricerca:",
   "Inizia ricerca",
   "Termina ricerca",
   "Server:",
   "Nome utente:",
   "Scarica files selezionati",
   "Elimina trasferimenti completi/errati",
   "Rimuovi selezione",
   "Chat",
   "Nome",
   "ID",
   "BeShare sta ascoltando sulla porta %i.  Scegli Collegati dal menu File per iniziare!",
   "Impossibile iniziare thread di condivisione server!  (Prova a far ripartire la rete, poi a rilanciare BeShare)",
   "Impossibile scaricare dall'utente ",
   ", nessuna informazione disponibile sulla connessione.",
   "Impossibile scaricare files da ",
   ", perché siamo entrambi dietro un firewall.",
   "Sessione di accettazione dello scaricamento per ",
   " non è riuscita a partire!",
   "Sessione scaricamento file su ",
   "Ricerca cambiata in",
   "Ricerca attivata",
   "Impossibile iniziare sessione ShareFileTransfer per rispondere a richiesta di trasferimento remoto!  (Prova a far ripartire la rete, poi a rilanciare BeShare)",
   "Il tuo nome e' stato cambiato in ",
   "di",
   "Utente ignoto ",
   ", messaggio \"",
   "\" non inviato.",
   "Nessun utente specificato in /msg",
   "Ricerca interrotta",
   "Sistema",
   "Attenzione",
   "Errore",
   "Azione",
   "Collegati a ",
   " - Collegamento a ",
   " - %lu risultati mostrati (",
   " - Connesso a ",
   "Connessione stabilita.",
   "Collegamento al server...",
   "Non sei più connesso al server MUSCLE.",
   "Connessione al server fallita.",
   "Inglese",
   "Spagnolo",
   "Lingua %s selezionata.  La modifica avrà effetto quando rilancerai BeShare.",
   "Lingua", 
   "Tedesco",
   "Tipo",
   "Dimensione File",
   "Files",
   "Olandese",
   "Sto esaminando i files...",
   "Comandi tipo IRC disponibili:",
   "invia un messaggio privato",
   "cambia il tuo nome utente",
   "fai qualcosa (azione)",
   "sinonimo per /action",
   "cancella il chat log",
   "inizia una nuova ricerca",
   "interrompi la ricerca corrente",
   "collegati al server",
   "disconnettiti dal server",
   "esci da BeShare",
   "mostra questo testo d'aiuto",
   "<nome o id sessione> <testo>",
   "<nome>",
   "<azione>",
   "[stringaRicerca]",
   "[nomeServer]",
   "Portoghese",
   "Francese",
   "Suona Quando Menzionato",
   "Max Caricamenti Simultanei per Utente",
   "Mostra Tempi",
   "Elimina Automaticamente Scaricamenti Completati",
   "Max Scaricamenti Simultanei per Utente",
   "Italiano",
   "Collegati al Lancio",
   "Messaggi Privati",
   "Apri Finestra Privata di Chat",
   "BeShare Finestra Privata di Chat",
   "Chatta Con:",
   "Errore, nessun utente specificato!",
   "Salva Impostazioni",
   "Ripristina Impostazioni",
   "Errore, comando ignoto",
   "Scrivi /help per la lista dei comandi disponibili.",
   " - (%lu-%lu) dei %lu risultati mostrati (",
   "Visualizza",
   "Messaggi Informativi",
   "Messaggi d'Avvertimento",
   "Messaggi d'Errore",
   "Log Abilitato",
   "Log Disabilitato",
};

int GetLanguage()
{
   return _language;
}

void SetLanguage(int wl)
{
   if ((wl >= 0)&&(wl < NUM_LANGUAGES)) _language = wl;
}

const char * GetLanguageName(int wl)
{
   switch(wl)
   {
      case LANGUAGE_ENGLISH:    return str(STR_ENGLISH);
      case LANGUAGE_SPANISH:    return str(STR_SPANISH);
      case LANGUAGE_GERMAN:     return str(STR_GERMAN);
      case LANGUAGE_DUTCH:      return str(STR_DUTCH);
      case LANGUAGE_PORTUGUESE: return str(STR_PORTUGUESE);
      case LANGUAGE_FRENCH:     return str(STR_FRENCH);
      case LANGUAGE_ITALIAN:    return str(STR_ITALIAN);
      case LANGUAGE_MALKOVITCH: return "Malkovitch";
      default:  MASSERT(false, "Bad language token!"); return 0;
   }
}

const char * str(int ws)
{
   const char ** base = NULL;
   switch(_language)
   {
      case LANGUAGE_ENGLISH:    base = _englishStrings;    break;
      case LANGUAGE_SPANISH:    base = _spanishStrings;    break;
      case LANGUAGE_GERMAN:     base = _germanStrings;     break;
      case LANGUAGE_DUTCH:      base = _dutchStrings;      break;
      case LANGUAGE_PORTUGUESE: base = _portugueseStrings; break;
      case LANGUAGE_FRENCH:     base = _frenchStrings;     break;
      case LANGUAGE_ITALIAN:    base = _italianStrings;    break;
      case LANGUAGE_MALKOVITCH: 
      {
         switch(ws)
         {
            case STR_FILE_NAME_KEY:  return "\0011Malkovitch";  // gotta make sure these are 
            case STR_USER_KEY:       return "\0012Malkovitch";  // prefixed correctly, even
            case STR_SESSIONID_KEY:  return "\0013Malkovitch";  // inside Malkovitch-land
            case STR_CONNECTION_KEY: return "\0014Malkovitch";
            default:                 return "Malkovitch";
         }
      }
      break;
   }
   return ((base)&&(ws >= 0)&&(ws < NUM_STRINGS)) ? base[ws] : NULL;
}

char shortcut(int sc)
{
   const char * base = NULL;
   switch(_language)
   {
      case LANGUAGE_ENGLISH:    base = _englishShortcuts;    break;
      case LANGUAGE_SPANISH:    base = _spanishShortcuts;    break;
      case LANGUAGE_GERMAN:     base = _germanShortcuts;     break;
      case LANGUAGE_DUTCH:      base = _dutchShortcuts;      break;
      case LANGUAGE_PORTUGUESE: base = _portugueseShortcuts; break;
      case LANGUAGE_FRENCH:     base = _frenchShortcuts;     break;
      case LANGUAGE_ITALIAN:    base = _italianShortcuts;    break;
      case LANGUAGE_MALKOVITCH: return 'M';
   }
   return ((base)&&(sc >= 0)&&(sc < NUM_SHORTCUTS)) ? base[sc] : 0;
}
