//    Peek, An Image Viewing Program for BeOS
//    Copyright (C) 2000 Durand Miller
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 2.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include "ImageView.h"
#include "PeekWindow.h"

ImageView::ImageView(BRect R,char* name, PeekConfiguration* s, BMessage *n) 
: BView(R,name,B_FOLLOW_ALL_SIDES,B_WILL_DRAW | B_FRAME_EVENTS)
{
  Setup = s;
  negotiationMessage = n;
  SetViewColor(0,0,0,0);
  imagePic = BTranslationUtils::GetBitmap(name);
  mouseDownFlag = false;
  clipRegion = NULL;
  mouseClipping = false;
  popup = new BPopUpMenu("popup");

  BMenu *tempItemB;

    
    popup->AddItem(new BMenuItem("Copy",new BMessage(PEEK_CLIPBOARD_COPY),'C'));
    popup->AddItem(new BMenuItem("Paste",new BMessage(PEEK_CLIPBOARD_PASTE),'V'));

  popup->AddSeparatorItem();

  BMenu *tempItemA;
   tempItemB= new BMenu("Set as Desktop");
      tempItemA = new BMenu("current");
        tempItemA->AddItem(new BMenuItem("Tiled", new BMessage(PEEK_DESKTOP_TILED)));
        tempItemA->AddItem(new BMenuItem("Centered", new BMessage(PEEK_DESKTOP_CENTERED)));
        tempItemA->AddItem(new BMenuItem("Scaled", new BMessage(PEEK_DESKTOP_SCALED)));
        tempItemA->AddItem(new BMenuItem("Manual Placement", new BMessage(PEEK_DESKTOP_MANUAL)));
      tempItemB->AddItem(tempItemA);
      tempItemA = new BMenu("all");
        tempItemA->AddItem(new BMenuItem("Tiled", new BMessage(PEEK_ALL_DESKTOP_TILED)));
        tempItemA->AddItem(new BMenuItem("Centered", new BMessage(PEEK_ALL_DESKTOP_CENTERED)));
        tempItemA->AddItem(new BMenuItem("Scaled", new BMessage(PEEK_ALL_DESKTOP_SCALED)));
        tempItemA->AddItem(new BMenuItem("Manual Placement", new BMessage(PEEK_ALL_DESKTOP_MANUAL)));
      tempItemB->AddItem(tempItemA);
    popup->AddItem(tempItemB);

  popup->AddSeparatorItem();

  tempItemB= new BMenu("Viewing Modes");
    tempItemB->SetRadioMode(true);
    tempItemB->AddItem(new BMenuItem("Normal", new BMessage(PEEK_IMAGE_NORMAL)));
    tempItemB->AddItem(new BMenuItem("Tiled", new BMessage(PEEK_IMAGE_TILED)));
    tempItemB->AddItem(new BMenuItem("Stretched to window", new BMessage(PEEK_IMAGE_STRETCH)));
    tempItemB->AddItem(new BMenuItem("Window resized to image", new BMessage(PEEK_IMAGE_RESIZE)));
  popup->AddItem(tempItemB);

  tempItemB= new BMenu("Slide Show Modes");
    tempItemB->SetRadioMode(true);
    tempItemB->AddItem(new BMenuItem("Descending", new BMessage(PEEK_SLIDE_DESCEND)));
    tempItemB->AddItem(new BMenuItem("Ascending", new BMessage(PEEK_SLIDE_ASCEND)));
    tempItemB->AddItem(new BMenuItem("Random", new BMessage(PEEK_SLIDE_RANDOM)));
  popup->AddItem(tempItemB);


}

ImageView::~ImageView() {
 if (imagePic) delete imagePic;
 if (currentRef) delete currentRef;
 delete popup;
}

// Attached to window
void ImageView::AttachedToWindow() {
 SetViewBitmap(imagePic);
 delete imagePic;
 imagePic = NULL;
 currentRef = NULL;
 BView::AttachedToWindow();
}

// adjust scroll bars - This just realigns the scrollbars and 
//                      stuff
void ImageView::AdjustScrollBars() {
 if (imagePic == NULL) return;
 float x,y;
 x = imagePic->Bounds().Width();
 y = imagePic->Bounds().Height();
 y = y - Bounds().Height();
 x = x - Bounds().Width();
 if (y < 0) y = 0;  if (x < 0) x = 0;
 if (Setup->ViewMode() != P_STRETCHED) {
   ScrollBar(B_VERTICAL)->SetRange(0,y);
   ScrollBar(B_HORIZONTAL)->SetRange(0,x);
 } else {
   ScrollBar(B_VERTICAL)->SetRange(0,0);
   ScrollBar(B_HORIZONTAL)->SetRange(0,0);
 }  
}


// Refresh - this invalidates the region around the picture's new location
//           and then only this part gets updated.  This helps cut out that
//           flashing we used to have when the frame was resized. 
void ImageView::Refresh() {
 if (imagePic != NULL && Setup->ViewMode() != P_TILED) {
   float imageWidth = imagePic->Bounds().right;
   float imageHeight = imagePic->Bounds().bottom;
   float left,top;
   if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
   if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);

   BRegion *viewRegion = new BRegion(Bounds());
   viewRegion->Exclude( BRect( left, top, left + imageWidth, top + imageHeight ));
   Invalidate(viewRegion);
   delete viewRegion;
 }

  AdjustScrollBars();
  Draw(Bounds());
}

// Draw
void ImageView::Draw(BRect R)  {
 if (imagePic == NULL) return;

 float imageWidth = imagePic->Bounds().right;
 float imageHeight = imagePic->Bounds().bottom;

 if (Setup->ViewMode() == P_TILED) 
   for (float left = 0; left < Bounds().Width(); left += imageWidth )
    for (float top = 0; top < Bounds().Height(); top += imageHeight ) {
     MovePenTo(left,top);
     DrawBitmap(imagePic);
    } 

 float left,top;
 if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
 if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);

 if (Setup->ViewMode() == P_NORMAL || Setup->ViewMode() == P_WINDOW) {
   MovePenTo(left,top);
   DrawBitmap(imagePic);
 }

 if (Setup->ViewMode() == P_STRETCHED) {
   DrawBitmap(imagePic,imagePic->Bounds(),Bounds());
 }

 if (clipRegion != NULL)  DrawClipping(NULL);

 Sync();
}


// size of image
void ImageView::RequestedSize(float *x, float *y) {
  if (!imagePic) {
   *x = 0;
   *y = 0;
  } else {
    *x = imagePic->Bounds().right;
    *y = imagePic->Bounds().bottom;
  }  
}

// GetBitmap() 
BBitmap* ImageView::GetBitmap() {
  return imagePic;
}

// GetFilename()
int ImageView::GetFilename(char* fileName) {
 if (imagePic == NULL) return B_ERROR;
 if (currentRef == NULL) return B_ERROR;
 BEntry *tempEntry = new BEntry(currentRef);
 BPath *tempPath = new BPath;
 tempEntry->GetPath(tempPath);
 strcpy(fileName,tempPath->Path());
 delete tempEntry;
 delete tempPath;
 return B_OK;
}

void ImageView::Clear() {
 if (imagePic) {
    delete imagePic;
    imagePic = NULL;
   }
 if (currentRef) {
    delete currentRef;
    currentRef = NULL;
   }
 if (clipRegion) {
   delete clipRegion;
   clipRegion = NULL;
   mouseClipping = false;
 }
}

// GetRef(entry_ref *)
int ImageView::GetRef(entry_ref* file) {
 if (currentRef == NULL) return B_ERROR;
 *file = *currentRef;
 return B_OK;
}

// loadImage (entry_ref)
int ImageView::loadImage(entry_ref fileRef) {
 Clear();
 Hide();
 imagePic = BTranslationUtils::GetBitmap(&fileRef);
 if (imagePic) {
   currentRef = new entry_ref(fileRef);
   Show();
   return B_OK;
 }  
 Show();
 return B_ERROR;
}

// loadImage (BEntry *)
int ImageView::loadImage(BEntry *e) {
  entry_ref *tempRef = new entry_ref;
  e->GetRef(tempRef);
  int k = loadImage(*tempRef);
  delete tempRef;
  return k;
}


// loadImage for BITMAPS
int ImageView::loadImage(BBitmap *src) {
 Clear();
 Hide();

 imagePic = src;
 if (imagePic) {
   Show();
   return B_OK;
 }  
 Show();
 return B_ERROR;
}

// Mouse down
void ImageView::MouseDown(BPoint a) {
  if (imagePic == NULL) return;
  BPoint dummy,imagePoint;
  uint32 button;
  GetMouse(&dummy,&button);

  uint32 task = P_SCROLLING;
  if (button == B_PRIMARY_MOUSE_BUTTON) task = Setup->LeftButton();
  if (button == B_SECONDARY_MOUSE_BUTTON) task = Setup->RightButton();
  if (button == B_TERTIARY_MOUSE_BUTTON) task = Setup->MiddleButton();

  if (task == P_SCROLLING) {
    mouseDownFlag = true;
    mousePosition = new BPoint(a);  
    SetMouseEventMask(1);   // Receive MouseMoved even outside of BView
    return;
  };

  if (task == P_DRAGGING) {  
    BBitmap *dragImage;
    BMessage *msg;

      
    // here comes the CTRL keyboard modifier
    if (modifiers() & B_CONTROL_KEY) {

      if (!Setup->FileDrop()) {
          if (currentRef==NULL) return;
          msg = new BMessage(B_SIMPLE_DATA);
          msg->AddRef("refs",currentRef);
       } else msg = new BMessage (*negotiationMessage);

    } else {

      if (Setup->FileDrop()) {
         if (currentRef==NULL) return;
         msg = new BMessage(B_SIMPLE_DATA);
         msg->AddRef("refs",currentRef);
      }  else msg = new BMessage (*negotiationMessage);

    }
       
    
    if (Setup->DragMode()!=B_OP_OVER) {
       if (imagePic->Bounds().Width() < Bounds().Width())  imagePoint.x = a.x - Bounds().Width()/2 +  imagePic->Bounds().Width()/2; else imagePoint.x = a.x;
       if (imagePic->Bounds().Height() < Bounds().Height()) imagePoint.y = a.y - Bounds().Height()/2 + imagePic->Bounds().Height()/2; else imagePoint.y = a.y;

       if (clipRegion == NULL) dragImage = new BBitmap(imagePic);
       else {
         dragImage = GetClipping();
         imagePoint.x = dragImage->Bounds().Width()/2;
         imagePoint.y = dragImage->Bounds().Height()/2;
        }

       DragMessage(msg,dragImage, Setup->DragMode(), imagePoint);
     } else {
        imagePoint.x = 32;
        imagePoint.y = 32;
        BMimeType k("image");          
        dragImage = new BBitmap(BRect(0,0,31,31),B_CMAP8);
        status_t h = k.GetIcon(dragImage,B_LARGE_ICON);
        if (h == B_OK) DragMessage(msg,dragImage, B_OP_ALPHA, imagePoint);
     }  
   
   delete msg;
   return;
  }

 if (task == P_CLIPPING && Setup->ViewMode() != P_STRETCHED && Setup->ViewMode() != P_TILED) {
   if (clipRegion) {
      DrawClipping(NULL);
      delete clipRegion;
      clipRegion = NULL;
    }  
   if (!InView(a)) return;
   if (imagePic == NULL) return;
   mouseClipping = true;
   clipRegion = new BRect;
   clipRegion->Set(a.x,a.y,a.x,a.y); 
   SetMouseEventMask(1);   // Receive MouseMoved even outside of BView
   return;
 }

 if (task == P_MENU) {
   dummy = a;
   ConvertToScreen(&dummy);
   BMenuItem *temp = popup->Go(dummy);
   if (temp == NULL) return;
   if (temp->Message() == NULL) return;
   Window()->PostMessage(temp->Message());
   return;
 }
 
}

bool ImageView::InView(BPoint a) {
 float imageWidth = imagePic->Bounds().Width();
 float imageHeight = imagePic->Bounds().Height();
 float left,top;
 if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
 if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);

 if (a.x > left && a.y > top && 
     a.x < imageWidth + left && a.y < imageHeight + top) return true;

 return false;
}

void ImageView::DrawClipping(BPoint *a = NULL) {
 if (clipRegion == NULL) return;

   float left = 0,top = 0;
   if (a == NULL) {
     float imageWidth = imagePic->Bounds().Width();
     float imageHeight = imagePic->Bounds().Height();
     if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
     if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);
   }

    BRect temp(clipRegion->left + left, clipRegion->top + top, clipRegion->right + left, clipRegion->bottom + top);

     SetDrawingMode(B_OP_INVERT);
     if (a !=NULL) {
       StrokeRect(temp, B_MIXED_COLORS);
       clipRegion->right = a->x;
       clipRegion->bottom = a->y;
       temp.right = a->x;
       temp.bottom = a->y;
     }  
     StrokeRect(temp, B_MIXED_COLORS);
     SetDrawingMode(B_OP_COPY);
}

// mouse up
void ImageView::MouseUp(BPoint a) {
  if (mouseDownFlag) {
    mouseDownFlag = false; 
    SetMouseEventMask(0);   // Stop the receive MouseMoved outside of BView
    delete mousePosition;
  }

  if (mouseClipping) {
     mouseClipping = false;
     if (clipRegion->Width() < 10 && clipRegion->Height() < 10) {
      SetDrawingMode(B_OP_INVERT);
      StrokeRect(*clipRegion, B_MIXED_COLORS);
      SetDrawingMode(B_OP_COPY);
      delete clipRegion;
      clipRegion = NULL;
      return;
     }
     float imageWidth = imagePic->Bounds().Width();
     float imageHeight = imagePic->Bounds().Height();
     float left,top;
     if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
     if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);
     float xx1 = clipRegion->left - left;
     float yy1 = clipRegion->top - top;
     clipRegion->Set( xx1,yy1, xx1+clipRegion->Width(), yy1+clipRegion->Height() );
  }
}

// mouse moved
void ImageView::MouseMoved(BPoint a, uint32 transit, const BMessage* e) {
  if (mouseDownFlag) {
   float hmin,hmax,vmin,vmax;
   ScrollBar(B_HORIZONTAL)->GetRange(&hmin,&hmax);  
   ScrollBar(B_VERTICAL)->GetRange(&vmin,&vmax);  
   float xdiff =  (mousePosition->x  - a.x) + ScrollBar(B_HORIZONTAL)->Value();
   float ydiff =  (mousePosition->y  - a.y) + ScrollBar(B_VERTICAL)->Value();
   if (xdiff <= hmin ) xdiff = hmin;
   if (ydiff <= vmin ) ydiff = vmin;   
   if (xdiff >= hmax ) xdiff = hmax;
   if (ydiff >= vmax ) ydiff = vmax;   
   ScrollTo(xdiff,ydiff);
  }

 if (mouseClipping) {
   if (!InView(a)) return;
   DrawClipping(&a);
  }
}


// supported format
bool ImageView::SupportedFormat(const char *s) {
      translator_id *list;
      int32 num;
      BTranslatorRoster::Default()->GetAllTranslators(&list,&num);

       for (int i=0; i < num; i++) {
         int32 numOfTypes;
         const translation_format *in_types;
         BTranslatorRoster::Default()->GetInputFormats(list[i],&in_types,&numOfTypes);
         for (int j = 0; j <numOfTypes; j++) if ( strcmp(in_types[j].MIME,s) == 0) return true;
       }
 return false;
}

// handle mime
void ImageView::HandleMime(BMessage *e) {
    int32 numBytes;
    type_code type;
    const void *data;
    const char *m_type;
    if (e->GetInfo( B_MIME_TYPE, 0, (char**)&m_type, &type, &numBytes) != B_OK) return;
    e->FindData(m_type,B_MIME_TYPE,&data,&numBytes);
    BMemoryIO memStream(data, numBytes);
    BBitmapStream imageStream;
    if (strcmp(m_type,"image/x-be-bitmap")==0) {
     uint8 buffer[1024];
     int32 size;
     while ((size=memStream.Read(buffer,1024))>0) 
       imageStream.Write(buffer,size);
    } 
    else if ( BTranslatorRoster::Default()->Translate(&memStream,NULL,NULL, &imageStream, B_TRANSLATOR_BITMAP, 0, m_type) != B_OK) return;

    BBitmap *bmp;
    imageStream.DetachBitmap(&bmp);
    loadImage(bmp);
    AdjustScrollBars();
    return;  
}

// choose format
void ImageView::ChooseFormat(BMessage *e) {
                                           // CASE 1:  we have a file
     entry_ref ref;
     if (e->FindRef("refs", &ref) == B_OK )  {

       // here, i'm using a method from PeekWindow - LoadImage.  I'm doing this because
       // the selection thread must be disabled if a ref is loaded and selected in the list
       // or else it loads twice and flashes on the screen. PeekWindow takes care of that.

       if (((PeekWindow*)Window()) -> LoadImage(&ref) == B_ERROR) { // if it doesn't work, it might be a directory
         BEntry *sam = new BEntry(&ref,true);
         if (sam->IsDirectory()) ((PeekWindow*)Window()) -> ChangeDirectory(&ref);
         delete sam;
       }

       
      return;
     }

     const char *m_type;
     int32 i = 0, action;
     while ( e->FindInt32("be:actions",i,&action) == B_OK && action != B_COPY_TARGET) i++;
     if (action != B_COPY_TARGET) return;
     i = 0;
     while (e->FindString("be:types",i,&m_type) == B_OK && !SupportedFormat(m_type)) i++;
     if (!m_type) return; 
     
     BMessage yahoo(B_COPY_TARGET);
     yahoo.AddString("be:types",m_type);
     e->SendReply(&yahoo,this);
}

// message received
void ImageView::MessageReceived(BMessage *e) {
  switch (e->what) {
    case B_MIME_DATA: HandleMime(e); break;
    case B_COPY_TARGET: ActionCopy(e); break;
    case B_TRASH_TARGET: Trash(); break;
    case B_SIMPLE_DATA: ChooseFormat(e); break;
    default: BView::MessageReceived(e);
  }
}

// trash
void ImageView::Trash() {
  ((PeekWindow*)Window())->DeleteFile();
}

// GetClipping - returns the clipped region if there is one. If there isn't, it returns the whole BBitmap in memory - if there is one;
BBitmap * ImageView::GetClipping() {
    if (clipRegion == NULL) {
      if (imagePic == NULL) return NULL;
      BBitmap *dragImage = new BBitmap(imagePic);
      return dragImage;
    }
    BBitmap *dragImage = new BBitmap(BRect(0,0,clipRegion->Width(),clipRegion->Height()), imagePic-> ColorSpace(),true,false);
    BView *tempView = new BView(dragImage->Bounds(),"yo",0,0);
    dragImage->AddChild(tempView);
    dragImage->Lock();
    tempView->MovePenTo( -clipRegion->left, -clipRegion->top);
    tempView->DrawBitmap(imagePic);
    dragImage->Unlock();
    dragImage->RemoveChild(tempView);
    delete tempView;
    return dragImage;
}

void ImageView::ActionCopy(BMessage *e) {   
  entry_ref targetRef;
  // memory or file transfer ?
  if (e->FindRef("directory",&targetRef) == B_OK)  {

      BDirectory *targetDir = new BDirectory( &targetRef );
      if (targetDir->InitCheck() == B_OK) {
         BFile *targetFile = new BFile(targetDir,e->FindString("name"),B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
         if (targetFile->InitCheck() == B_OK) {

          BBitmap *dragImage = NULL;
          BBitmapStream *strm;
          if (clipRegion== NULL) strm = new BBitmapStream(imagePic);         // clipping versus non-clipping
           else {
            dragImage = GetClipping();
            strm = new BBitmapStream(dragImage);
          }

          translator_id trans_id;
          uint32 type;

          GetTranslatorInfo(e->FindString("be:filetypes"),&trans_id,&type);
          BTranslatorRoster::Default()->Translate(trans_id,strm,NULL,targetFile,type);
          BNodeInfo *ni = new BNodeInfo(targetFile);
          ni->SetType(e->FindString("be:filetypes"));
          if (dragImage == NULL) strm->DetachBitmap(&imagePic);
           else {
            strm->DetachBitmap(&dragImage);
            delete dragImage;
           }
          delete ni;
          delete strm;
         } 
        delete targetFile;
       }   
     delete targetDir;
     return;
   } 
   
  else {
     BMallocIO *data = new BMallocIO;
     BBitmap *dragImage;
     BBitmapStream *imageSrc;
     if (clipRegion == NULL) imageSrc = new BBitmapStream(imagePic);
     else {
       dragImage = GetClipping();
       imageSrc = new BBitmapStream(dragImage);
     }
     BMessage *package = new BMessage(B_MIME_DATA);
     translator_id trans_id;
     uint32 type;

     GetTranslatorInfo(e->FindString("be:types"),&trans_id,&type);
     if (BTranslatorRoster::Default()->Translate( trans_id, imageSrc, NULL, data, type ) != B_OK) return;
     package->AddData( e->FindString("be:types"),  B_MIME_DATA, data->Buffer(), data->BufferLength());
     e->SendReply(package);        

     if (clipRegion == NULL) imageSrc->DetachBitmap(&imagePic);
      else {
       imageSrc->DetachBitmap(&dragImage);
       delete dragImage;
      }
     delete data;
     delete imageSrc;
     delete package;
  } 

}

void ImageView::ClipboardCopy() {
  BMessage *clip = NULL;
  if (imagePic == NULL) return;  // nothing to copy

  if (be_clipboard->Lock()) {
      be_clipboard->Clear();
      if ((clip = be_clipboard->Data())) {

        BBitmap* clipPic = NULL;                      // clip versus whole image
        if (clipRegion == NULL) clipPic = imagePic;
        else clipPic = GetClipping();

        clip->AddString("class","BBitmap");
        clip->AddRect("_frame",clipPic->Bounds());
        clip->AddInt32("_cspace",clipPic->ColorSpace());
        clip->AddInt32("_bmflags",0);                    // hi, what flags are these? thanks
        clip->AddInt32("_rowbytes",clipPic->BytesPerRow());
        clip->AddData("_data",B_RAW_TYPE,clipPic->Bits(), clipPic->BitsLength());
        be_clipboard->Commit();

        if (clipRegion != NULL) delete clipPic;  // if was clip, free some memory.

      }
     be_clipboard->Unlock();
  }
}

void ImageView::ClipboardPaste() {
  BMessage *clip = NULL;

  const void *buffer;
  int32 bufferLength;

  if (be_clipboard->Lock()) {
      if ((clip = be_clipboard->Data())) {
       if (strcmp(clip->FindString("class"),"BBitmap")==0) {
          Clear();
          imagePic = new BBitmap( clip->FindRect("_frame"), (color_space) clip->FindInt32("_cspace") );
          clip->FindData("_data",B_RAW_TYPE,(const void **)&buffer, &bufferLength);
          memcpy(imagePic->Bits(), buffer, bufferLength);
          Refresh();
        }
      }
     be_clipboard->Unlock();
  }
}


void ImageView::GetTranslatorInfo(const char *m, translator_id *t, uint32 *t2) {
      translator_id *list;
      int32 num;
      BTranslatorRoster::Default()->GetAllTranslators(&list,&num);

       for (int i=0; i < num; i++) {
         int32 numOfTypes;
         const translation_format *out_types;
         BTranslatorRoster::Default()->GetOutputFormats(list[i],&out_types,&numOfTypes);
         for (int j = 0; j <numOfTypes; j++) if ( strcmp(out_types[j].MIME,m) == 0) {
           *t = list[i];
           *t2 = out_types[j].type;
           return;
         }
       }
 *t = 0;
 *t2 = 0;
}
