//  ImageView.h
//
//  Created by Durand Miller, durandm@attglobal.net
//
//
//  This will be the main image viewing view class with the
//  picture.
//


#ifndef _IMAGE_VIEW_H
#define _IMAGE_VIEW_H

#include <View.h>
#include <Bitmap.h>
#include <TranslationUtils.h>
#include <Entry.h>
#include <ScrollBar.h>
#include <Message.h>
#include "constants.h"
#include "PeekConfiguration.h"
#include <Mime.h>
#include <Path.h>
#include <string.h>
#include <Directory.h>
#include <TranslatorRoster.h>
#include <BitmapStream.h>
#include <NodeInfo.h>
#include <Window.h>
#include <Picture.h>
#include <Resources.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Region.h>

class ImageView : public BView {
  public:
   ~ImageView();
   ImageView(BRect,char*,PeekConfiguration*,BMessage*);
   virtual void Draw(BRect);
   virtual void AttachedToWindow();
   void RequestedSize(float *x, float *y);
   int loadImage(entry_ref);
   int loadImage(BBitmap *);
   int loadImage(BEntry *);
   BBitmap *GetBitmap();
   BBitmap *GetClipping();
   int GetFilename(char*);
   int GetRef(entry_ref *);
   void Clear();
   void Refresh();
   void AdjustScrollBars();
   void ClipboardPaste();
   void ClipboardCopy();
  private:
   virtual void MouseDown(BPoint);
   virtual void MouseUp(BPoint);
   virtual void MouseMoved(BPoint, uint32, const BMessage*);   
   virtual void MessageReceived(BMessage *);
   BPopUpMenu *popup;
   bool InView(BPoint);
   void DrawClipping(BPoint *);
   bool SupportedFormat(const char*);
   void GetTranslatorInfo(const char*, translator_id*, uint32*);
   void Trash();
   void HandleMime(BMessage *);
   void ChooseFormat(BMessage *);
   void ActionCopy(BMessage *);
   BRect *clipRegion;
   BMessage *negotiationMessage;
   PeekConfiguration* Setup;
   entry_ref *currentRef;
   BPoint *mousePosition;
   bool mouseDownFlag, mouseClipping;
   BBitmap *imagePic;
};


#endif