//    Peek, An Image Viewing Program for BeOS
//    Copyright (C) 2000 Durand Miller
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 2.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include "OptionsWindow.h"


OptionsWindow::OptionsWindow(BRect R,char* name, PeekConfiguration *s) : 
BWindow(R,name,B_TITLED_WINDOW,0,B_CURRENT_WORKSPACE) {
  Setup = s;
  BRect *tempRect = new BRect(0,0,0,0);
  
  *tempRect = Bounds();

  BTabView* now = new BTabView(*tempRect,"options_tabview");

  SlideView = new SlideOptions(*tempRect,"Slide show");
  MouseView = new MouseOptions(*tempRect,"Mouse");
  ViewView = new ViewOptions(*tempRect,"Viewing");
  FileView = new FileOptions(*tempRect,"Files & Misc");

  now->AddTab(ViewView);
  now->AddTab(SlideView);
  now->AddTab(MouseView);
  now->AddTab(FileView);
  AddChild(now);

// Setting up the slideshow view
  if (Setup->SlideshowMode()==P_DESCEND) SlideView->Flip(SLIDE_FORWARD);
  if (Setup->SlideshowMode()==P_ASCEND) SlideView->Flip(SLIDE_BACKWARD);
  if (Setup->SlideshowMode()==P_RANDOM) SlideView->Flip(SLIDE_RANDOM);
  SlideView->SetSliderValue(Setup->SlideshowDelay());
  SlideView->Change();
// done
  
// Setting up the mouse view
  if (Setup->LeftButton()==P_CLIPPING) MouseView->Change(MOUSE_LEFT_CLIP);
  if (Setup->LeftButton()==P_DRAGGING) MouseView->Change(MOUSE_LEFT_DRAG);
  if (Setup->LeftButton()==P_SCROLLING) MouseView->Change(MOUSE_LEFT_SCROLL);
  if (Setup->LeftButton()==P_MENU) MouseView->Change(MOUSE_LEFT_MENU);

  if (Setup->RightButton()==P_CLIPPING) MouseView->Change(MOUSE_RIGHT_CLIP);
  if (Setup->RightButton()==P_DRAGGING) MouseView->Change(MOUSE_RIGHT_DRAG);
  if (Setup->RightButton()==P_SCROLLING) MouseView->Change(MOUSE_RIGHT_SCROLL);
  if (Setup->RightButton()==P_MENU) MouseView->Change(MOUSE_RIGHT_MENU);

  if (Setup->MiddleButton()==P_CLIPPING) MouseView->Change(MOUSE_MIDDLE_CLIP);
  if (Setup->MiddleButton()==P_DRAGGING) MouseView->Change(MOUSE_MIDDLE_DRAG);
  if (Setup->MiddleButton()==P_SCROLLING) MouseView->Change(MOUSE_MIDDLE_SCROLL);
  if (Setup->MiddleButton()==P_MENU) MouseView->Change(MOUSE_MIDDLE_MENU);
  
  if (Setup->DragMode()==B_OP_BLEND) MouseView->Change(MOUSE_DRAG_BLEND);
  if (Setup->DragMode()==B_OP_COPY) MouseView->Change(MOUSE_DRAG_COPY);
  if (Setup->DragMode()==B_OP_INVERT) MouseView->Change(MOUSE_DRAG_INVERT);
  if (Setup->DragMode()==B_OP_OVER) MouseView->Change(MOUSE_DRAG_ICON);  
// done
  if (Setup->ViewMode()==P_NORMAL) ViewView->Change(VIEW_MODE_NORMAL);
  if (Setup->ViewMode()==P_TILED) ViewView->Change(VIEW_MODE_TILED);
  if (Setup->ViewMode()==P_STRETCHED) ViewView->Change(VIEW_MODE_STRETCH);
  if (Setup->ViewMode()==P_WINDOW) ViewView->Change(VIEW_MODE_WINDOW);

  if (Setup->FullScreen()) ViewView->Change(VIEW_MODE_FULL_SCREEN);
  if (Setup->FileListHide()) ViewView->Change(VIEW_MODE_FILE_LIST);

  if (Setup->FileDrop()) MouseView->Change(MOUSE_FILE_DROP);
  if (Setup->ChangeDirectory()) MouseView->Change(MOUSE_FILE_CHANGE);

  if (Setup->RunFile()) FileView->Change(FILE_RUN_FILE);
  if (Setup->DeleteAll()) FileView->Change(FILE_DELETE_ALL);
  if (Setup->ChangeOnDesktopLoad()) FileView->Change(FILE_CHANGE_ON_DESKTOP_LOAD);
  if (Setup->OpenCompact()) FileView->Change(FILE_OPEN_COMPACT);

  delete tempRect;
}

void OptionsWindow::SetMouse(uint32 msg) {
  if ( msg == MOUSE_LEFT_CLIP ) Setup->SetLeftButton(P_CLIPPING);
  if ( msg == MOUSE_LEFT_DRAG ) Setup->SetLeftButton(P_DRAGGING);
  if ( msg == MOUSE_LEFT_SCROLL ) Setup->SetLeftButton(P_SCROLLING);
  if ( msg == MOUSE_LEFT_MENU ) Setup->SetLeftButton(P_MENU);
  // right
  if ( msg == MOUSE_RIGHT_CLIP ) Setup->SetRightButton(P_CLIPPING);
  if ( msg == MOUSE_RIGHT_DRAG ) Setup->SetRightButton(P_DRAGGING);
  if ( msg == MOUSE_RIGHT_SCROLL ) Setup->SetRightButton(P_SCROLLING);
  if ( msg == MOUSE_RIGHT_MENU ) Setup->SetRightButton(P_MENU);
  //middle
  if ( msg == MOUSE_MIDDLE_CLIP ) Setup->SetMiddleButton(P_CLIPPING);
  if ( msg == MOUSE_MIDDLE_DRAG ) Setup->SetMiddleButton(P_DRAGGING);
  if ( msg == MOUSE_MIDDLE_SCROLL ) Setup->SetMiddleButton(P_SCROLLING);
  if ( msg == MOUSE_MIDDLE_MENU ) Setup->SetMiddleButton(P_MENU);

  //drag view
  if ( msg == MOUSE_DRAG_BLEND ) Setup->SetDragMode(B_OP_BLEND);
  if ( msg == MOUSE_DRAG_COPY ) Setup->SetDragMode(B_OP_COPY);
  if ( msg == MOUSE_DRAG_INVERT ) Setup->SetDragMode(B_OP_INVERT);
  if ( msg == MOUSE_DRAG_ICON ) Setup->SetDragMode(B_OP_OVER);
}

void OptionsWindow::MessageReceived(BMessage *e) {
  uint32 msg = e->what;
  if (
    msg == MOUSE_LEFT_CLIP ||
    msg == MOUSE_LEFT_DRAG ||
    msg == MOUSE_LEFT_SCROLL ||
    msg == MOUSE_LEFT_MENU ||
    msg == MOUSE_RIGHT_CLIP ||
    msg == MOUSE_RIGHT_DRAG ||
    msg == MOUSE_RIGHT_SCROLL ||
    msg == MOUSE_RIGHT_MENU ||
    msg == MOUSE_MIDDLE_CLIP ||
    msg == MOUSE_MIDDLE_DRAG ||
    msg == MOUSE_MIDDLE_SCROLL ||
    msg == MOUSE_MIDDLE_MENU ||
    msg == MOUSE_DRAG_BLEND ||
    msg == MOUSE_DRAG_COPY ||
    msg == MOUSE_DRAG_INVERT ||
    msg == MOUSE_DRAG_ICON 
     )  { 
      MouseView->Change(msg);
      SetMouse(msg);
      return;
     } 

  if (
       msg == VIEW_MODE_NORMAL ||
       msg == VIEW_MODE_TILED ||
       msg == VIEW_MODE_WINDOW ||
       msg == VIEW_MODE_STRETCH 
     ) {
      if (msg == VIEW_MODE_NORMAL) Setup->SetViewMode(P_NORMAL);
      if (msg == VIEW_MODE_TILED) Setup->SetViewMode(P_TILED);
      if (msg == VIEW_MODE_STRETCH) Setup->SetViewMode(P_STRETCHED);
      if (msg == VIEW_MODE_WINDOW) Setup->SetViewMode(P_WINDOW);
      ViewView->Change(msg);
      return;
     }

  if ( msg == VIEW_MODE_FULL_SCREEN ) {
       be_app->PostMessage(PEEK_FULL_SCREEN);
       return;
      }

  if ( msg == VIEW_MODE_FILE_LIST ) {
       be_app->PostMessage(PEEK_FILES_HIDE);
       return;
      }

  if (msg == MOUSE_FILE_DROP) Setup->SetFileDrop(!Setup->FileDrop());
  if (msg == MOUSE_FILE_CHANGE) Setup->SetChangeDirectory(!Setup->ChangeDirectory());
  if (msg == FILE_RUN_FILE) Setup->SetRunFile(!Setup->RunFile());
  if (msg == FILE_DELETE_ALL) Setup->SetDeleteAll(!Setup->DeleteAll());
  if (msg == FILE_CHANGE_ON_DESKTOP_LOAD) Setup->SetChangeOnDesktopLoad(!Setup->ChangeOnDesktopLoad());
  if (msg == FILE_OPEN_COMPACT) Setup->SetOpenCompact(!Setup->OpenCompact());

  switch (msg) {
    case SLIDE_CHANGE: Setup->SetSlideshowDelay(SlideView->SliderValue()); SlideView->Change(); break;
    case SLIDE_FORWARD: SlideView->Flip(e->what); Setup->SetSlideshowMode(P_DESCEND); break;
    case SLIDE_BACKWARD: SlideView->Flip(e->what); Setup->SetSlideshowMode(P_ASCEND);  break;
    case SLIDE_RANDOM: SlideView->Flip(e->what); Setup->SetSlideshowMode(P_RANDOM);  break;
    default: BWindow::MessageReceived(e);
  }  
}




void OptionsWindow::Quit() {
  be_app->PostMessage(PEEK_OPTIONS_CLOSE);
   BWindow::Quit();
}