//  PeekWindow.h
//
//  Main window setup for Peek :)
//
//  Created by Durand Miller, durandm@attglobal.net
//


#ifndef _PEEK_WINDOW_H
#define _PEEK_WINDOW_H

#include <Application.h>
#include <Window.h>
#include "constants.h"
#include "ImageView.h"
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <ScrollView.h>
#include <Directory.h>
#include <Path.h>
#include <Entry.h>
#include <Screen.h>
#include <OS.h>
#include <stdlib.h>
#include <Roster.h>
#include <strings.h>
#include "PeekConfiguration.h"
#include <FindDirectory.h>
#include <Box.h>
#include <Tracker/Background.h>
#include <TranslatorRoster.h>
#include <FilePanel.h>
#include <String.h>
#include <NodeInfo.h>
#include <NodeMonitor.h>
#include <fs_attr.h>
#include <Alert.h>
#include "PListItem.h"
#include "PListView.h"
#include "SlideView.h"

class PeekWindow : public BWindow {
  public:
   PeekWindow(BRect, char*, PeekConfiguration *);
   bigtime_t SlideshowDelay();
   int32 EntrySelected();
   void ConfirmSelection(int32);
   int32 LoadImage(BEntry *);
   int32 LoadImage(int32);
   int32 LoadImage(entry_ref *);
   void ToggleSlideshow();
   void SliderMoved(float);
   void ChangeDirectory(entry_ref *);
   void ToggleFileList();
   void DeleteFile();
   void RefreshFileView();

 private:
   virtual bool QuitRequested();
   virtual void MessageReceived(BMessage*);
   virtual void FrameResized(float,float);
   virtual void FrameMoved(BPoint);


   void CleanListView(PListView *);
   BMenu *MakeSaveMenu(char*);
   BMessage *negotiationMessage;
   BFilePanel *saveWindow;
   PeekConfiguration *Setup;
   void ToggleFullScreen();

   bool ImageType(BEntry *);
   void SetDesktopBackground(int32);
   void SetAllDesktopBackground(int32);
   void GetDesktopBackground();
   void CaptureScreen(bool);

   void SaveFile(BMessage *);
   void FileSave(BMessage *);

   void WaitASec();
   BView *alles;
   SlideView *sliderBar;
   void ShowHTML();
   thread_id slides, selectThread;
   bool slideMode;
   void ResizeToImageView();
   void Invoked();
   BMenuBar *menuBar;
   ImageView *picView;
   BPath *currentPath;
   PListView *fileView;
   BScrollView *imageScrollView, *fileScrollView;
   int32 lastSlid;
 
};

#endif